@@ -169,6 +169,14 @@ def check_complex(config, mathlibs):
     priv = []
     pub = []
 
+    try:
+        if os.uname()[0] == "Interix":
+            warnings.warn("Disabling broken complex support. See #1365")
+            return priv, pub
+    except:
+        # os.uname not available on all platforms. blanket except ugly but safe
+        pass
+
     # Check for complex support
     st = config.check_header('complex.h')
     if st:
@@ -215,12 +223,15 @@ def check_ieee_macros(config):
     _macros = ["isnan", "isinf", "signbit", "isfinite"]
     if sys.version_info[:2] >= (2, 6):
         for f in _macros:
-            st = config.check_decl(fname2def("decl_%s" % f),
+            py_symbol = fname2def("decl_%s" % f)
+            already_declared = config.check_decl(py_symbol,
                     headers=["Python.h", "math.h"])
-            if not st:
-                macros.append(f)
+            if already_declared:
+                if config.check_macro_true(py_symbol,
+                        headers=["Python.h", "math.h"]):
+                    pub.append('NPY_%s' % fname2def("decl_%s" % f))
             else:
-                _add_decl(f)
+                macros.append(f)
     else:
         macros = _macros[:]
     # Normally, isnan and isinf are macro (C99), but some platforms only have
@@ -422,7 +433,7 @@ def configuration(parent_package='',top_path=None):
                 PYTHON_HAS_UNICODE_WIDE = True
             else:
                 PYTHON_HAS_UNICODE_WIDE = False
-                
+
             if ENABLE_SEPARATE_COMPILATION:
                 moredefs.append(('ENABLE_SEPARATE_COMPILATION', 1))
 
@@ -589,7 +600,10 @@ def configuration(parent_package='',top_path=None):
 
         subpath = join('src', 'multiarray')
         sources = [join(local_dir, subpath, 'scalartypes.c.src'),
-                   join(local_dir, subpath, 'arraytypes.c.src')]
+                   join(local_dir, subpath, 'arraytypes.c.src'),
+                   join(local_dir, subpath, 'nditer.c.src'),
+                   join(local_dir, subpath, 'lowlevel_strided_loops.c.src'),
+                   join(local_dir, subpath, 'einsum.c.src')]
 
         # numpy.distutils generate .c from .c.src in weird directories, we have
         # to add them there as they depend on the build_dir
@@ -607,6 +621,8 @@ def configuration(parent_package='',top_path=None):
         from numpy.distutils.misc_util import get_cmd
 
         subpath = join('src', 'umath')
+        # NOTE: For manual template conversion of loops.h.src, read the note
+        #       in that file.
         sources = [join(local_dir, subpath, 'loops.c.src'),
                    join(local_dir, subpath, 'umathmodule.c.src')]
 
@@ -657,7 +673,7 @@ def configuration(parent_package='',top_path=None):
                                   generate_numpyconfig_h,
                                   generate_numpy_api,
                                   ],
-                         )
+                         libraries=['npymath'])
 
     # npymath needs the config.h and numpyconfig.h files to be generated, but
     # build_clib cannot handle generate_config_h and generate_numpyconfig_h
@@ -689,13 +705,14 @@ def configuration(parent_package='',top_path=None):
             sources=[join('src', 'npymath', 'npy_math.c.src'),
                      join('src', 'npymath', 'ieee754.c.src'),
                      join('src', 'npymath', 'npy_math_complex.c.src'),
+                     join('src', 'npymath', 'halffloat.c'),
                      get_mathlib_info],
             install_dir='lib')
     config.add_npy_pkg_config("npymath.ini.in", "lib/npy-pkg-config",
             subst_dict)
     config.add_npy_pkg_config("mlib.ini.in", "lib/npy-pkg-config",
             subst_dict)
-   
+
     multiarray_deps = [
             join('src', 'multiarray', 'arrayobject.h'),
             join('src', 'multiarray', 'arraytypes.h'),
@@ -721,13 +738,15 @@ def configuration(parent_package='',top_path=None):
             join('src', 'multiarray', 'sequence.h'),
             join('src', 'multiarray', 'shape.h'),
             join('src', 'multiarray', 'ucsnarrow.h'),
-            join('src', 'multiarray', 'usertypes.h')]
+            join('src', 'multiarray', 'usertypes.h'),
+            join('src', 'private', 'lowlevel_strided_loops.h')]
 
     multiarray_src = [join('src', 'multiarray', 'multiarraymodule.c'),
         join('src', 'multiarray', 'hashdescr.c'),
         join('src', 'multiarray', 'arrayobject.c'),
         join('src', 'multiarray', 'numpymemoryview.c'),
         join('src', 'multiarray', 'buffer.c'),
+        join('src', 'multiarray', 'datetime.c'),
         join('src', 'multiarray', 'numpyos.c'),
         join('src', 'multiarray', 'conversion_utils.c'),
         join('src', 'multiarray', 'flagsobject.c'),
@@ -749,7 +768,12 @@ def configuration(parent_package='',top_path=None):
         join('src', 'multiarray', 'scalarapi.c'),
         join('src', 'multiarray', 'refcount.c'),
         join('src', 'multiarray', 'arraytypes.c.src'),
-        join('src', 'multiarray', 'scalartypes.c.src')]
+        join('src', 'multiarray', 'scalartypes.c.src'),
+        join('src', 'multiarray', 'nditer.c.src'),
+        join('src', 'multiarray', 'lowlevel_strided_loops.c.src'),
+        join('src', 'multiarray', 'dtype_transfer.c'),
+        join('src', 'multiarray', 'nditer_pywrap.c'),
+        join('src', 'multiarray', 'einsum.c.src')]
 
     if PYTHON_HAS_UNICODE_WIDE:
         multiarray_src.append(join('src', 'multiarray', 'ucsnarrow.c'))
@@ -766,7 +790,7 @@ def configuration(parent_package='',top_path=None):
         multiarray_deps.extend(multiarray_src)
         multiarray_src = [join('src', 'multiarray', 'multiarraymodule_onefile.c')]
         multiarray_src.append(generate_multiarray_templated_sources)
- 
+
         umath_deps.extend(umath_src)
         umath_src = [join('src', 'umath', 'umathmodule_onefile.c')]
         umath_src.append(generate_umath_templated_sources)
@@ -798,6 +822,7 @@ def configuration(parent_package='',top_path=None):
                                   generate_numpyconfig_h,
                                   generate_numpy_api,
                                   generate_ufunc_api],
+                         libraries=['npymath'],
                          )
 
     # Configure blasdot
