@@ -1,4 +1,5 @@
 import sys
+import platform
 
 from numpy.testing import *
 import numpy.core.umath as ncu
@@ -10,14 +11,19 @@ import numpy as np
 
 # At least on Windows the results of many complex functions are not conforming
 # to the C99 standard. See ticket 1574.
-err = np.seterr(invalid='ignore')
+# Ditto for Solaris (ticket 1642) and OS X on PowerPC.
+olderr = np.seterr(invalid='ignore', divide='ignore')
 try:
-    functions_seem_flaky = (np.exp(complex(np.inf, 0)).imag != 0)
+    functions_seem_flaky = ((np.exp(complex(np.inf, 0)).imag != 0)
+                            or (np.log(complex(np.NZERO, 0)).imag != np.pi))
 finally:
-    np.seterr(**err)
+    np.seterr(**olderr)
 # TODO: replace with a check on whether platform-provided C99 funcs are used
-have_platform_functions = (sys.platform == 'win32')
-skip_complex_tests = have_platform_functions and functions_seem_flaky
+have_platform_functions = (sys.platform.startswith('sunos') or
+                           (sys.platform == 'darwin' and 'powerpc' in
+                                             platform.processor()))
+skip_complex_tests = (sys.platform.startswith('win') or
+                      (have_platform_functions and functions_seem_flaky))
 
 def platform_skip(func):
     return dec.skipif(skip_complex_tests,
@@ -148,12 +154,7 @@ class TestClog(TestCase):
         for i in range(len(x)):
             assert_almost_equal(y[i], y_r[i])
 
-    #@platform_skip
-    # Clog tests have been failing for a long time on Windows, so always skip
-    # on that platform. In 1.5.x the platform_skip decorator is Windows-only
-    # anyway.
-    @dec.knownfailureif(sys.platform == 'win32',
-                        "clog(- inf + i inf) fails on Windows.")
+    @platform_skip
     def test_special_values(self):
         xl = []
         yl = []
@@ -248,11 +249,7 @@ class TestClog(TestCase):
         # clog(- inf + i inf) returns +inf + i3pi /4.
         x = np.array([complex(-np.inf, np.inf)], dtype=np.complex)
         y = np.complex(np.inf, 0.75 * np.pi)
-        err = np.seterr(invalid="ignore")
-        try:
-            assert_almost_equal(np.log(x), y)
-        finally:
-            np.seterr(**err)
+        assert_almost_equal(np.log(x), y)
         xl.append(x)
         yl.append(y)
 
@@ -437,13 +434,15 @@ class TestCabs(object):
     def test_simple(self):
         x = np.array([1+1j, 0+2j, 1+2j, np.inf, np.nan])
         y_r = np.array([np.sqrt(2.), 2, np.sqrt(5), np.inf, np.nan])
-        err = np.seterr(invalid="ignore")
-        y = np.abs(x)
+
+        olderr = np.seterr(invalid='ignore')
         try:
-            for i in range(len(x)):
-                assert_almost_equal(y[i], y_r[i])
+            y = np.abs(x)
         finally:
-            np.seterr(**err)
+            np.seterr(**olderr)
+
+        for i in range(len(x)):
+            assert_almost_equal(y[i], y_r[i])
 
     def test_fabs(self):
         # Test that np.abs(x +- 0j) == np.abs(x) (as mandated by C99 for cabs)
@@ -453,12 +452,12 @@ class TestCabs(object):
         x = np.array([complex(1, np.NZERO)], dtype=np.complex)
         assert_array_equal(np.abs(x), np.real(x))
 
-        x = np.array([complex(np.inf, np.NZERO)], dtype=np.complex)
-        err = np.seterr(invalid="ignore")
+        olderr = np.seterr(invalid='ignore')
         try:
+            x = np.array([complex(np.inf, np.NZERO)], dtype=np.complex)
             assert_array_equal(np.abs(x), np.real(x))
         finally:
-            np.seterr(**err)
+            np.seterr(**olderr)
 
         x = np.array([complex(np.nan, np.NZERO)], dtype=np.complex)
         assert_array_equal(np.abs(x), np.real(x))
